@@ -22,7 +22,7 @@
 Kaldi engine classes
 """
 
-import collections, os, subprocess, sys, threading, time
+import collections, logging, os, subprocess, sys, threading, time
 
 from packaging.version import Version
 from six import integer_types, string_types, print_, reraise
@@ -33,7 +33,8 @@ from kaldi_active_grammar       import KaldiAgfNNet3Decoder, KaldiError
 from ..base                     import (EngineBase, EngineError, MimicFailure,
                                         DelegateTimerManager,
                                         DelegateTimerManagerInterface,
-                                        DictationContainerBase)
+                                        DictationContainerBase,
+                                        GrammarWrapperBase)
 from .audio                     import MicAudio, VADAudio, AudioStore, WavAudio
 from .recobs                    import KaldiRecObsManager
 from .testing                   import debug_timer
@@ -86,6 +87,13 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
             self._log.error("See https://dragonfly2.readthedocs.io/en/latest/kaldi_engine.html#updating-to-a-new-version")
             raise EngineError("Incompatible kaldi_active_grammar version")
 
+        # Hack to avoid bug processing keyboard actions on Windows
+        if os.name == 'nt':
+            action_exec_logger = logging.getLogger('action.exec')
+            if action_exec_logger.getEffectiveLevel() > logging.DEBUG:
+                self._log.warning("%s: Enabling logging of actions execution to avoid bug processing keyboard actions on Windows", self)
+                action_exec_logger.setLevel(logging.DEBUG)
+
         if not (isinstance(retain_dir, string_types) or (retain_dir is None)):
             self._log.error("Invalid retain_dir: %r" % retain_dir)
             retain_dir = None
@@ -187,7 +195,8 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
             self.connect()
 
         kaldi_rule_by_rule_dict = self._compiler.compile_grammar(grammar, self)
-        wrapper = GrammarWrapper(grammar, kaldi_rule_by_rule_dict, self)
+        wrapper = GrammarWrapper(grammar, kaldi_rule_by_rule_dict, self,
+                                 self._recognition_observer_manager)
         for kaldi_rule in kaldi_rule_by_rule_dict.values():
             kaldi_rule.load(lazy=self._compiler.lazy_compilation)
 
@@ -259,6 +268,9 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
     def _get_language(self):
         return "en"
 
+    def _has_quoted_words_support(self):
+        return False
+
     def prepare_for_recognition(self):
         """ Can be called optionally before ``do_recognition()`` to speed up its starting of active recognition. """
         self._compiler.prepare_for_recognition()
@@ -317,7 +329,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
                         self.audio_store.add_block(block)
                     output, likelihood = self._decoder.get_output()
                     self._log.log(5, "Partial phrase: likelihood %f, %r [in_complex=%s]", likelihood, output, in_complex)
-                    kaldi_rule, words, words_are_dictation, in_dictation = self._compiler.parse_partial_output(output)
+                    kaldi_rule, words, words_are_dictation_mask, in_dictation = self._compiler.parse_partial_output(output)
                     in_complex = bool(in_dictation or (kaldi_rule and kaldi_rule.is_complex))
 
                 else:
@@ -445,7 +457,10 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
                     if not mimic:
                         # We should never receive an unparsable recognition from kaldi, only from mimic
                         self._log.error("unable to parse recognition: %r" % output)
-                    self._recognition_observer_manager.notify_failure()
+
+                    # FIXME
+                    results_obj = None
+                    self._recognition_observer_manager.notify_failure(results_obj)
                     return None, ''
                 if len(results) > 1:
                     self._log.warning("ambiguity in recognition: %r" % output)
@@ -453,16 +468,19 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
                     results.sort(key=lambda result: 100 if result[0].has_dictation else 0)
 
                 kaldi_rule, words = results[0]
-                words_are_dictation = [True] * len(words)  # FIXME: hack, but seems to work fine? only a problem for ambiguous rules containing dictation, which should be handled above
+                words_are_dictation_mask = [True] * len(words)  # FIXME: hack, but seems to work fine? only a problem for ambiguous rules containing dictation, which should be handled above
 
         elif self._compiler.parsing_framework == 'token':
-            kaldi_rule, words, words_are_dictation = self._compiler.parse_output(output,
+            kaldi_rule, words, words_are_dictation_mask = self._compiler.parse_output(output,
                 dictation_info_func=lambda: (self.audio_store.current_audio_data, self._decoder.get_word_align(output)))
             if kaldi_rule is None:
                 if words != []:
                     # We should never receive an unparsable recognition from kaldi, unless it's empty (from noise)
                     self._log.error("unable to parse recognition: %r" % output)
-                self._recognition_observer_manager.notify_failure()
+
+                # FIXME
+                results_obj = None
+                self._recognition_observer_manager.notify_failure(results_obj)
                 return None, ''
 
             if self._log.isEnabledFor(12):
@@ -477,7 +495,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
         words = tuple(words)
         grammar_wrapper = self._get_grammar_wrapper(kaldi_rule.parent_grammar)
         with debug_timer(self._log.debug, "dragonfly parse time"):
-            grammar_wrapper.recognition_callback(words, kaldi_rule.parent_rule, words_are_dictation)
+            grammar_wrapper.recognition_callback(words, kaldi_rule.parent_rule, words_are_dictation_mask)
 
         parsed_output = ' '.join(words)
         return kaldi_rule, parsed_output
@@ -485,12 +503,12 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
 #===========================================================================
 
-class GrammarWrapper(object):
+class GrammarWrapper(GrammarWrapperBase):
 
-    def __init__(self, grammar, kaldi_rule_by_rule_dict, engine):
-        self.grammar = grammar
+    def __init__(self, grammar, kaldi_rule_by_rule_dict, engine,
+                 recobs_manager):
+        GrammarWrapperBase.__init__(self, grammar, engine, recobs_manager)
         self.kaldi_rule_by_rule_dict = kaldi_rule_by_rule_dict
-        self.engine = engine
 
         self.active = True
         self.exclusive = False
@@ -498,17 +516,22 @@ class GrammarWrapper(object):
     def phrase_start_callback(self, fg_window):
         self.grammar.process_begin(fg_window.executable, fg_window.title, fg_window.handle)
 
-    def recognition_callback(self, words, rule, words_are_dictation):
+    def recognition_callback(self, words, rule, words_are_dictation_mask):
         try:
             # Prepare the words and rule names for the element parsers
-            rule_names = (rule.name,) + (('dgndictation',) if any(words_are_dictation) else ())
+            rule_names = (rule.name,) + (('dgndictation',) if any(words_are_dictation_mask) else ())
             words_rules = tuple((word, 0 if not is_dictation else 1)
-                for (word, is_dictation) in zip(words, words_are_dictation))
+                for (word, is_dictation) in zip(words, words_are_dictation_mask))
+
+            # FIXME
+            results_obj = None
 
             # Attempt to parse the recognition
             func = getattr(self.grammar, "process_recognition", None)
             if func:
-                if not func(words):
+                if not self._process_grammar_callback(func, words=words,
+                                                      results=results_obj):
+                    # Return early if the method didn't return True or equiv.
                     return
 
             state = State(words_rules, rule_names, self.engine)
@@ -516,10 +539,11 @@ class GrammarWrapper(object):
             for result in rule.decode(state):
                 if state.finished():
                     root = state.build_parse_tree()
-                    self.engine._recognition_observer_manager.notify_recognition(words, rule, root)
+                    notify_args = (words, rule, root, results_obj)
+                    self.recobs_manager.notify_recognition(*notify_args)
                     with debug_timer(self.engine._log.debug, "rule execution time"):
                         rule.process_recognition(root)
-                    self.engine._recognition_observer_manager.notify_post_recognition(words, rule, root)
+                    self.recobs_manager.notify_post_recognition(*notify_args)
                     return
 
         except Exception as e:
