@@ -291,5 +291,55 @@ class EngineBase(object):
         """
         return self._get_language()
 
+    def _get_language_tag(self, language_id):
+        # Get a language tag from the Windows language identifier.
+        tags = self._language_tags.get(language_id)
+        if tags:
+            return tags[0]
+
+        # The _language_tags dictionary didn't contain the language, so
+        # get the best match by using the primary language identifier.
+        # This allows us to match unlisted language variants.
+        primary_id = language_id & 0x00ff
+        for language_id2, (tag, _) in self._language_tags.items():
+            if primary_id == language_id2 & 0x00ff:  # Match found.
+                return tag
+
+        # Speaker language wasn't found.
+        self._log.error("Unknown speaker language: 0x%04x" % language_id)
+        raise EngineError("Unknown speaker language: 0x%04x" % language_id)
+
+    _language_tags = {
+                      0x0c09: ("en", "AustralianEnglish"),
+                      0xf00a: ("es", "CastilianSpanish"),
+                      0xf809: ("en", "CAEnglish"),
+                      0x0004: ("zh", "Chinese"),
+                      0x0413: ("nl", "Dutch"),
+                      0x0009: ("en", "English"),
+                      0x040c: ("fr", "French"),
+                      0x0407: ("de", "German"),
+                      0xf009: ("en", "IndianEnglish"),
+                      0x0410: ("it", "Italian"),
+                      0x0411: ("jp", "Japanese"),
+                      0xf40a: ("es", "LatinAmericanSpanish"),
+                      0x0416: ("pt", "Portuguese"),
+                      0xf409: ("en", "SingaporeanEnglish"),
+                      0x040a: ("es", "Spanish"),
+                      0x0809: ("en", "UKEnglish"),
+                      0x0409: ("en", "USEnglish"),
+                     }
+
     def _get_language(self):
         raise NotImplementedError("Engine %s not implemented." % self)
+
+    @property
+    def quoted_words_support(self):
+        """
+        Whether this engine can compile and recognize quoted words.
+
+        :rtype: bool
+        """
+        return self._has_quoted_words_support()
+
+    def _has_quoted_words_support(self):
+        raise NotImplementedError("Engine %s not implemented." % self)
